==> udmsearch-3.0.10.tar.gz: udmsearch-3.0.10/src/parseurl.c <==
#include "udm_config.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "udm_common.h"
#include "udm_parseurl.h"
#include "udm_utils.h"


int freeurl(UDM_URL *url){
	if(url){
		UDM_FREE(url->schema);
		UDM_FREE(url->specific);
		UDM_FREE(url->hostinfo);
		UDM_FREE(url->hostname);
		UDM_FREE(url->path);
		UDM_FREE(url->filename);
		UDM_FREE(url->anchor);
	}
	return(0);
}

/* Oh,no! We have to remove parent level like /parent/../index.html from path
   Let's do it recursively! */
char *remove2dot(char *path){
    char *ptr;
    char *tail;
    
    if(!(ptr=strstr(path,"../"))) return path;
    if(ptr==path) return path; /* How could it be? */
    tail=ptr+2;
    ptr--;
    *ptr=0;
    if(!(ptr=strrchr(path,'/'))) *path=0; else *ptr=0;
    path=strcat(path,tail);
    return remove2dot(path);
}

int parseurl(UDM_URL *url,char *s){
char *ss;
int len;

	len=strlen(s)+1;
	url->schema=(char*)malloc(len);*(url->schema)=0;
	url->specific=(char*)malloc(len);*(url->specific)=0;
	url->hostinfo=(char*)malloc(len);*(url->hostinfo)=0;
	url->hostname=(char*)malloc(len);*(url->hostname)=0;
	url->anchor=(char*)malloc(len);*(url->anchor)=0;
	url->port=0;
	url->path=(char*)malloc(len);*(url->path)=0;
	url->filename=(char*)malloc(len);*(url->filename)=0;

	if(sscanf(s,"%[^:]:%s",url->schema,url->specific)!=2){
		strcpy(url->path,s);
		strcpy(url->schema,"");
		strcpy(url->specific,"");
	}else{
		if(!strcmp(url->schema,"ftp")||
		(!strcmp(url->schema,"http"))||
		(!strcmp(url->schema,"https"))||
		(!strcmp(url->schema,"news"))||
		(!strcmp(url->schema,"nntp"))){
			switch(sscanf(url->specific,"//%[^/]%s",url->hostinfo,url->path)){
				case 1: strcpy(url->path,"/");break;
				case 2: break;
				default:freeurl(url);return(1);
			}
			sscanf(url->hostinfo,"%[^:]:%d",url->hostname,&(url->port));
		}else
		if(!strcmp(url->schema,"file")){
			strcpy(url->path,url->specific);
		}else
		if(!strcmp(url->schema,"htdb")){
			strcpy(url->path,url->specific);
		}else{
			freeurl(url);
			return(1);
		}
	}

	if((ss=strstr(url->path,"#")))*ss=0;
	url->path=remove2dot(url->path);
	if(url->path[0]!='/'){
		if(!strncmp(url->path,"./",2))
			strcpy(url->filename,url->path+2);
		else
			strcpy(url->filename,url->path);
		strcpy(url->path,"");
	}
	if((ss=strrchr(url->path,'/'))&&(strcmp(ss,"/"))){
		strcpy(url->filename,ss+1);
		*(ss+1)=0;
	}
	return(0);
}


int freetag(UDM_TAG *tag){
	UDM_FREE(tag->name);
	UDM_FREE(tag->href);
	UDM_FREE(tag->src);
	UDM_FREE(tag->content);
	UDM_FREE(tag->equiv);
	UDM_FREE(tag->value);
	UDM_FREE(tag->selected);
	return(0);
}

int parsetag(UDM_TAG *tag,char *stag){
char *s,*e,*p;
int len;

	len=strlen(stag);s=stag+1; e=s;
	while(!strchr(" \t\r\n>",*e))e++; 
	*e=0;
	tag->name=strdup(s);
	tag->href=NULL;
	tag->src=NULL;
	tag->content=NULL;
	tag->equiv=NULL;
	tag->value=NULL;
	tag->selected=NULL;
	e++;
	UDM_SKIP(e," \t\r\n>");
	while(e-stag<len){
		s=e;
		UDM_SKIPN(e," \t\r\n=>");
		if(*e!='='){
			*e++=0;
			UDM_SKIP(e," \t\r\n>");
		}
		if(*e=='='){
			*e=0;e++;
			UDM_SKIP(e," \t\r\n>");
			p=e;
			if(*p=='"'){
				p++;e++;
				UDM_SKIPN(e,"\">");
			}else
			if(*p=='\''){
				p++;e++;
				UDM_SKIPN(e,"'>");
			}else{
				UDM_SKIPN(e," \t\r\n>");
			}
			*e=0;e++;

			if(!strcasecmp(s,"href")){
				tag->href=strdup(p);
			}else
			if(!strcasecmp(s,"value")){
				tag->value=strdup(p);
			}else
			if(!strcasecmp(s,"selected")){
				tag->selected=strdup(p);
			}else
			if(!strcasecmp(s,"src")){
				tag->src=strdup(p);
			}else
			if(!strcasecmp(s,"content")){
				tag->content=strdup(p);
			}else
			if(!strcasecmp(s,"name")||!strcasecmp(s,"http-equiv")){
				tag->equiv=strdup(p);
			}
		}
	}return(0);
}

==> udmsearch-3.0.11.tar.gz: udmsearch-3.0.11/src/parseurl.c <==
#include "udm_config.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "udm_common.h"
#include "udm_parseurl.h"
#include "udm_utils.h"


int UdmFreeURL(UDM_URL *url){
	if(url){
		UDM_FREE(url->schema);
		UDM_FREE(url->specific);
		UDM_FREE(url->hostinfo);
		UDM_FREE(url->hostname);
		UDM_FREE(url->path);
		UDM_FREE(url->filename);
		UDM_FREE(url->anchor);
	}
	return(0);
}

int UdmParseURL(UDM_URL *url,char *s){
char *ss;
int len;

	len=strlen(s)+1;
	url->schema=(char*)malloc(len);*(url->schema)=0;
	url->specific=(char*)malloc(len);*(url->specific)=0;
	url->hostinfo=(char*)malloc(len);*(url->hostinfo)=0;
	url->hostname=(char*)malloc(len);*(url->hostname)=0;
	url->anchor=(char*)malloc(len);*(url->anchor)=0;
	url->port=0;
	url->path=(char*)malloc(len);*(url->path)=0;
	url->filename=(char*)malloc(len);*(url->filename)=0;

	if(sscanf(s,"%[^:]:%s",url->schema,url->specific)!=2){
		strcpy(url->path,s);
		strcpy(url->schema,"");
		strcpy(url->specific,"");
	}else{
		if(!strcmp(url->schema,"ftp")||
		(!strcmp(url->schema,"http"))||
		(!strcmp(url->schema,"https"))||
		(!strcmp(url->schema,"news"))||
		(!strcmp(url->schema,"nntp"))){
			switch(sscanf(url->specific,"//%[^/]%s",url->hostinfo,url->path)){
				case 1: strcpy(url->path,"/");break;
				case 2: break;
				default:UdmFreeURL(url);return(1);
			}
			sscanf(url->hostinfo,"%[^:]:%d",url->hostname,&(url->port));
		}else
		if(!strcmp(url->schema,"file")){
			strcpy(url->path,url->specific);
		}else
		if(!strcmp(url->schema,"htdb")){
			strcpy(url->path,url->specific);
		}else{
			UdmFreeURL(url);
			return(1);
		}
	}

	if((ss=strstr(url->path,"#")))*ss=0;
	url->path=UdmRemove2Dot(url->path);
	if(url->path[0]!='/'){
		if(!strncmp(url->path,"./",2))
			strcpy(url->filename,url->path+2);
		else
			strcpy(url->filename,url->path);
		strcpy(url->path,"");
	}
	if((ss=strrchr(url->path,'/'))&&(strcmp(ss,"/"))){
		strcpy(url->filename,ss+1);
		*(ss+1)=0;
	}
	return(0);
}


/*********************** Tags ****************************/

int UdmFreeTag(UDM_TAG *tag){
	UDM_FREE(tag->name);
	UDM_FREE(tag->href);
	UDM_FREE(tag->src);
	UDM_FREE(tag->content);
	UDM_FREE(tag->equiv);
	UDM_FREE(tag->value);
	UDM_FREE(tag->selected);
	return(0);
}

int UdmParseTag(UDM_TAG *tag,char *stag){
char *s,*e,*p;
int len;

	len=strlen(stag);s=stag+1; e=s;
	while(!strchr(" \t\r\n>",*e))e++; 
	*e=0;
	tag->name=strdup(s);
	tag->href=NULL;
	tag->src=NULL;
	tag->content=NULL;
	tag->equiv=NULL;
	tag->value=NULL;
	tag->selected=NULL;
	e++;
	UDM_SKIP(e," \t\r\n>");
	while(e-stag<len){
		s=e;
		UDM_SKIPN(e," \t\r\n=>");
		if(*e!='='){
			*e++=0;
			UDM_SKIP(e," \t\r\n>");
		}
		if(*e=='='){
			*e=0;e++;
			UDM_SKIP(e," \t\r\n>");
			p=e;
			if(*p=='"'){
				p++;e++;
				UDM_SKIPN(e,"\">");
			}else
			if(*p=='\''){
				p++;e++;
				UDM_SKIPN(e,"'>");
			}else{
				UDM_SKIPN(e," \t\r\n>");
			}
			*e=0;e++;

			if(!strcasecmp(s,"href")){
				tag->href=strdup(p);
			}else
			if(!strcasecmp(s,"value")){
				tag->value=strdup(p);
			}else
			if(!strcasecmp(s,"selected")){
				tag->selected=strdup(p);
			}else
			if(!strcasecmp(s,"src")){
				tag->src=strdup(p);
			}else
			if(!strcasecmp(s,"content")){
				tag->content=strdup(p);
			}else
			if(!strcasecmp(s,"name")||!strcasecmp(s,"http-equiv")){
				tag->equiv=strdup(p);
			}
		}
	}return(0);
}
